const codes = require('./codes');
// eslint-disable-next-line no-unused-vars
const { LucidLog } = require('lucid-log');

let _SpellCheckProvider_supportedList = new WeakMap();
let _SpellCheckProvider_logger = new WeakMap();
let _SpellCheckProvider_window = new WeakMap();
class SpellCheckProvider {
	/**
	 * @param {Electron.BrowserWindow} window 
	 * @param {LucidLog} logger;
	 */
	constructor(window, logger) {
		_SpellCheckProvider_logger.set(this, logger);
		_SpellCheckProvider_window.set(this, window);
		init(this, window);
	}

	/**
	 * @type {Array<{language:string,code:string}>}
	 */
	get supportedList() {
		return _SpellCheckProvider_supportedList.get(this);
	}

	/**
	 * @type {Array<{key:string,list:{language:string,code:string}}>}
	 */
	get supportedListByGroup() {
		var groupedList = [];
		for (const language of this.supportedList) {
			var key = language.language.substring(0, 1);
			addLanguageToGroup(groupedList, key, language);
		}
		return groupedList;
	}

	/**
	 * @type {Electron.BrowserWindow}
	 */
	get window() {
		return _SpellCheckProvider_window.get(this);
	}

	/**
	 * @type {LucidLog}
	 */
	get logger() {
		return _SpellCheckProvider_logger.get(this);
	}

	isLanguageSupported(code) {
		return this.supportedList.some(i => {
			return i.code === code;
		});
	}

	/**
	 * @param {Array<string>} codes 
	 * @returns {Array<string>}
	 */
	setLanguages(codes) {
		const setlanguages = [];
		for (const c of codes) {
			if (!this.isLanguageSupported(c)) {
				this.logger.warn(`Unsupported language code '${c}' for spellchecker`);
			} else {
				setlanguages.push(c);
			}
		}
		this.window.webContents.session.setSpellCheckerLanguages(setlanguages);
		if (setlanguages.length > 0) {
			this.logger.debug(`Language codes ${setlanguages.join(',')} set for spellchecker`);
		} else {
			this.logger.debug('Spellchecker is disabled!');
		}

		return setlanguages;
	}
}


/**
 * @param {SpellCheckProvider} intance
 * @param {Electron.BrowserWindow} window 
 */
function init(intance, window) {
	const listFromElectron = window.webContents.session.availableSpellCheckerLanguages;
	var list = codes.filter(lf => {
		return listContains(listFromElectron, lf.code);
	});
	sortLanguages(list);
	_SpellCheckProvider_supportedList.set(intance, list);
}

/**
 * 
 * @param {Array<string>} list 
 * @param {string} text 
 */
function listContains(list, text) {
	return list.some(l => {
		return l === text;
	});
}

/**
 * @param {Array<{key:string,list:{language:string,code:string}}>} groupedList 
 * @param {string} key 
 * @param {{language:string,code:string}} language 
 */
function addLanguageToGroup(groupedList, key, language) {
	const group = groupedList.filter(f => f.key === key)[0];
	if (group) {
		group.list.push(language);
	} else {
		groupedList.push({
			key: key,
			list: [language]
		});
	}
}

/**
 * @param {Array<{language:string,code:string}} languages 
 */
function sortLanguages(languages) {
	languages.sort((a, b) => {
		return stringCompare(a.language.toLocaleLowerCase(), b.language.toLocaleLowerCase());
	});
}

function stringCompare(str1, str2) {
	const le = str1 < str2;
	const gr = str1 > str2;
	return le ? -1 : gr ? 1 : 0;
}

module.exports = { SpellCheckProvider };
